-----------------------------------------------------------------------------

local borderData =
{
	["NONE"] 		= { texture = "", size = 0, inset = 0 },
	["STANDARD"] 	= { texture = "Interface\\Tooltips\\UI-Tooltip-Border", size = 16, inset = 5 },
	["SQUARE"]	 	= { texture = "Interface\\DialogFrame\\UI-DialogBox-Border", size = 16, inset = 5 },
}

-----------------------------------------------------------------------------

local AloftFrame = Aloft:NewModule("Frame", "AceEvent-2.0")
AloftFrame.db = Aloft:AcquireDBNamespace("frame")
AloftFrame.healthBarDB = Aloft:AcquireDBNamespace("healthBar")
Aloft:RegisterDefaults("frame", "profile", {
	enableMouse		= true,
	borderStyle		= "NONE",
	borderColor		= { 1, 1, 1, 1 },
	width			= 10,
	height			= 10,
	packingWidth	= 10,
	packingHeight	= 10,
	backgroundAlpha	= 0,
	colorFormat		= "",
	backgroundColors =
	{
		friendlyPlayer	= { 0, 0, 0 },
		friendlyPet		= { 0, 0, 0 },
		friendlyNPC		= { 0, 0, 0 },
		friendlyBoss	= { 0, 0, 0 },
		hostilePlayer	= { 0, 0, 0 },
		hostilePet		= { 0, 0, 0 },
		hostileNPC		= { 0, 0, 0 },
		hostileBoss		= { 0, 0, 0 },
		neutral			= { 0, 0, 0 },
		pet				= { 0, 0, 0 },
		groupPet		= { 0, 0, 0 },
	},
})

-----------------------------------------------------------------------------

local colorMethod = nil
local colorMethodData = { }
local frameUpdateList = { }

-----------------------------------------------------------------------------

function AloftFrame:UpdateAll()
	for aloftData in Aloft:IterateNameplates() do
		self:SetupFrame(aloftData)
	end
	
	for aloftData in Aloft:IterateVisibleNameplates() do
		if aloftData.backdropFrame then
			self:OnNameplateShowWithColor(aloftData)
		else
			self:OnNameplateShow(aloftData)
		end
	end
end

function AloftFrame:RegisterEvents()
	self:UnregisterAllEvents()
	self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
	self:RegisterEvent("Aloft:OnNameplateHide", "OnNameplateHide")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "OnCombatFinished")

	local profile = self.db.profile
	local onShowMethod = "OnNameplateShow"
	
	if (profile.backgroundAlpha ~= 0) and
	   (not self:IsArrayEqual(profile.backgroundColors.friendlyPlayer, profile.backgroundColors.friendlyPet) or
		not self:IsArrayEqual(profile.backgroundColors.friendlyPlayer, profile.backgroundColors.friendlyNPC) or
		not self:IsArrayEqual(profile.backgroundColors.friendlyPlayer, profile.backgroundColors.friendlyBoss) or
		not self:IsArrayEqual(profile.backgroundColors.friendlyPlayer, profile.backgroundColors.hostilePlayer) or
		not self:IsArrayEqual(profile.backgroundColors.friendlyPlayer, profile.backgroundColors.hostilePet) or
		not self:IsArrayEqual(profile.backgroundColors.friendlyPlayer, profile.backgroundColors.hostileNPC) or
		not self:IsArrayEqual(profile.backgroundColors.friendlyPlayer, profile.backgroundColors.hostileBoss) or
		not self:IsArrayEqual(profile.backgroundColors.friendlyPlayer, profile.backgroundColors.neutral) or
		not self:IsArrayEqual(profile.backgroundColors.friendlyPlayer, profile.backgroundColors.pet) or
		not self:IsArrayEqual(profile.backgroundColors.friendlyPlayer, profile.backgroundColors.groupPet) or
		profile.colorByClass or
		profile.colorHostileByClass or
		profile.colorFormat ~= "") then
		
		if not profile.colorByClass and
		   not profile.colorHostileByClass and
		   (self:IsArrayEqual(profile.backgroundColors.friendlyNPC, profile.backgroundColors.friendlyPet) and
			self:IsArrayEqual(profile.backgroundColors.friendlyNPC, profile.backgroundColors.friendlyBoss) and
			self:IsArrayEqual(profile.backgroundColors.friendlyNPC, profile.backgroundColors.pet) and
			self:IsArrayEqual(profile.backgroundColors.friendlyNPC, profile.backgroundColors.groupPet)) and
		   (self:IsArrayEqual(profile.backgroundColors.hostilePlayer, profile.backgroundColors.hostilePet) and
			self:IsArrayEqual(profile.backgroundColors.hostilePlayer, profile.backgroundColors.hostileNPC) and
			self:IsArrayEqual(profile.backgroundColors.hostilePlayer, profile.backgroundColors.hostileBoss)) then
			onShowMethod = "OnNameplateShowWithColorByType"
		else		
			onShowMethod = "OnNameplateShowWithColor"
		end
	end

	self:RegisterEvent("Aloft:OnNameplateShow", onShowMethod)

	if profile.backgroundAlpha ~= 0 then	
		if profile.colorByClass or profile.colorHostileByClass then
			self:RegisterEvent("Aloft:OnClassDataChanged", onShowMethod)
		end

		if not self:IsArrayEqual(profile.backgroundColors.friendlyPet, profile.backgroundColors.friendlyNPC) or
		   not self:IsArrayEqual(profile.backgroundColors.hostilePet, profile.backgroundColors.hostileNPC) then
			self:RegisterEvent("Aloft:OnIsPetDataChanged", onShowMethod)
		end
		for events in pairs(colorMethodData.events) do
			self:RegisterEvent(events, onShowMethod)
		end
	end
	
end

-----------------------------------------------------------------------------

function AloftFrame:OnEnable()
	self:UpdateAll()
end

function AloftFrame:OnDisable()
	self:UnregisterAllEvents()

	for aloftData in Aloft:IterateNameplates() do
		local backdropFrame = aloftData.backdropFrame
		if backdropFrame then
			backdropFrame:Hide()
		end
	end
end

local dataRequiredList = { }
function AloftFrame:RequiresData()
	local profile = self.db.profile

	colorMethodData = Aloft:CreateTag(profile.colorFormat)
	colorMethod = colorMethodData.method
	
	self:RegisterEvents()

	for i = 1,#dataRequiredList do
		dataRequiredList[i] = nil
	end

	if profile.backgroundAlpha ~= 0 then
		if profile.colorByClass or
		   profile.colorHostileByClass or
		   not self:IsArrayEqual(profile.backgroundColors.hostileNPC, profile.backgroundColors.hostilePlayer) then
			table.insert(dataRequiredList, "class")
		end

		if not self:IsArrayEqual(profile.backgroundColors.friendlyPet, profile.backgroundColors.friendlyNPC) or
		   not self:IsArrayEqual(profile.backgroundColors.hostilePet, profile.backgroundColors.hostileNPC) then
			table.insert(dataRequiredList, "isPet")
		end

		if Aloft:IsDataAvailable("unitid") and not self:IsArrayEqual(profile.backgroundColors.friendlyPet, profile.backgroundColors.groupPet) then
			table.insert(dataRequiredList, "unitid")
		end

		for data in pairs(colorMethodData.data) do
			table.insert(dataRequiredList, data)
		end
	end

	return unpack(dataRequiredList)
end

local backdropTable =
{
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground", tile = true, tileSize = 16,
	insets = { }
}

function AloftFrame:SetupFrame(aloftData)
	--
	-- Using the inbuilt frame for borders/background works fine, until a nameplate shows during combat
	--
	-- The game likes to automatically resize the frame when a frame is shown, 
	-- but we can't change it back OnShow because of in-combat lockdown.
	--
	-- This forces us to use our own backdropFrame which we can control as we please.
	--
	-- We also can't use SetFrameLevel during combat.
	--
	-- Requirements:
	--  Backdrop must appear behind all elements
	--  CastBar/HealthBar must be show backgrounds
	--  CastBar/HealthBar background must be above backdrop
	--  Overlay must display on top of HealthBar/CastBar
	--  UI elements must appear on top of healthbar/castbar/overlay
	--
	local profile = self.db.profile
	local backdropFrame = aloftData.backdropFrame

	if InCombatLockdown() then
		frameUpdateList[aloftData] = true
	else
		aloftData.nameplateFrame:EnableMouse(not self.db.profile.enableMouse)
	end

	if profile.borderStyle == "NONE" and profile.backgroundAlpha == 0 then
		if backdropFrame then
			backdropFrame:Hide()
		end
		return
	end
	
	if not backdropFrame then
		local nameplateFrame = aloftData.nameplateFrame
		backdropFrame = CreateFrame("Frame", nil, nameplateFrame)
		backdropFrame:SetPoint("CENTER", nameplateFrame, "CENTER")
		backdropFrame:SetAlpha(nameplateFrame:GetAlpha())
		backdropFrame:SetFrameLevel(0)
		aloftData.backdropFrame = backdropFrame
	end

	local data = borderData[profile.borderStyle]
	backdropTable.edgeFile		= data.texture
	backdropTable.edgeSize		= data.size
	backdropTable.insets.left	= data.inset
	backdropTable.insets.right	= data.inset
	backdropTable.insets.top	= data.inset
	backdropTable.insets.bottom = data.inset
	
	backdropFrame:SetBackdrop(backdropTable)
	backdropFrame:SetBackdropBorderColor(unpack(profile.borderColor))
	
	local healthBarProfile = self.healthBarDB.profile
	backdropFrame:SetWidth(profile.width+healthBarProfile.width)
	backdropFrame:SetHeight(profile.height+healthBarProfile.height)
	backdropFrame:Show()
	
	-- This shifts the backdrop to border layer, enabling it to display, despite being at frame level 0
	backdropFrame:GetRegions():SetDrawLayer("BORDER")

	self:SetBackgroundColor(aloftData, profile.backgroundColors.friendlyPlayer)
end

function AloftFrame:SetBackgroundColor(aloftData, color)
	aloftData.backdropFrame:SetBackdropColor(color[1], color[2], color[3], self.db.profile.backgroundAlpha)
end

function AloftFrame:SetBackgroundColorRgb(aloftData, color)
	aloftData.backdropFrame:SetBackdropColor(color.r, color.g, color.b, self.db.profile.backgroundAlpha)
end

function AloftFrame:OnNameplateShow(aloftData)
	if InCombatLockdown() then
		frameUpdateList[aloftData] = true
	else
		local profile = self.db.profile
		local healthBarProfile = self.healthBarDB.profile
		local nameplateFrame = aloftData.nameplateFrame

		nameplateFrame:SetWidth(profile.packingWidth+healthBarProfile.width)
		nameplateFrame:SetHeight(profile.packingHeight+healthBarProfile.height)
	end
end

function AloftFrame:OnNameplateHide(aloftData)
	frameUpdateList[aloftData] = nil
end

function AloftFrame:OnCombatFinished()
	for aloftData in pairs(frameUpdateList) do
		aloftData.nameplateFrame:EnableMouse(not self.db.profile.enableMouse)
		self:OnNameplateShow(aloftData)
		frameUpdateList[aloftData] = nil
	end
end

-----------------------------------------------------------------------------

function AloftFrame:ColourFriendlyPlayer(aloftData)
	if self.db.profile.colorByClass then
		local class = aloftData.class
		if class then
			local color = RAID_CLASS_COLORS[class]
			if color then
				self:SetBackgroundColorRgb(aloftData, color)
				return
			end
		end
	end
	self:SetBackgroundColor(aloftData, self.db.profile.backgroundColors.friendlyPlayer)
end

function AloftFrame:ColourFriendlyNPC(aloftData)
	-- Could be player pet, group pet, friendly pet, friendlyBoss, or friendlyNPC
	if aloftData.name == UnitName("pet") then
		self:SetBackgroundColor(aloftData, self.db.profile.backgroundColors.pet)
	elseif aloftData.unitid then
		self:SetBackgroundColor(aloftData, self.db.profile.backgroundColors.groupPet)
	elseif aloftData.isPet then
		self:SetBackgroundColor(aloftData, self.db.profile.backgroundColors.friendlyPet)
	elseif aloftData.isBoss then
		self:SetBackgroundColor(aloftData, self.db.profile.backgroundColors.friendlyBoss)
	else
		self:SetBackgroundColor(aloftData, self.db.profile.backgroundColors.friendlyNPC)
	end
end

function AloftFrame:ColourHostile(aloftData)
	-- player, hostile pet, hostile boss, hostileNPC
	if self.db.profile.colorHostileByClass then
		local class = aloftData.class
		if class then
			local color = RAID_CLASS_COLORS[class]
			if color then
				self:SetBackgroundColorRgb(aloftData, color)
				return
			end
		end
	end

	if aloftData.class then
		self:SetBackgroundColor(aloftData, self.db.profile.backgroundColors.hostilePlayer)
	elseif aloftData.isPet then
		self:SetBackgroundColor(aloftData, self.db.profile.backgroundColors.hostilePet)
	elseif aloftData.isBoss then
		self:SetBackgroundColor(aloftData, self.db.profile.backgroundColors.hostileBoss)
	else
		self:SetBackgroundColor(aloftData, self.db.profile.backgroundColors.hostileNPC)
	end
end

function AloftFrame:ColourNeutral(aloftData)
	self:SetBackgroundColor(aloftData, self.db.profile.backgroundColors.neutral)
end

function AloftFrame:ColourUnknown(aloftData)
end

local backgroundColorMethods =
{
	friendlyPlayer = AloftFrame.ColourFriendlyPlayer,
	friendlyNPC = AloftFrame.ColourFriendlyNPC,
	hostile = AloftFrame.ColourHostile,
	neutral = AloftFrame.ColourNeutral,
	unknown = AloftFrame.ColourUnknown,
}

function AloftFrame:OnNameplateShowWithColor(aloftData)
	self:OnNameplateShow(aloftData)
	local override = colorMethod(aloftData)
	if #override >= 6 then
		local rhex, ghex, bhex = override:sub(1, 2), override:sub(3, 4), override:sub(5, 6)
		aloftData.backdropFrame:SetBackdropColor(tonumber(rhex, 16)/255, tonumber(ghex, 16)/255, tonumber(bhex, 16)/255, self.db.profile.backgroundAlpha)
	else
		backgroundColorMethods[aloftData.type](self, aloftData)
	end
end

local backgroundTypeColorLookup =
{
	friendlyPlayer = "friendlyPlayer",
	friendlyNPC = "friendlyNPC",
	hostile = "hostileNPC",
	neutral = "neutral",
	unknown = "friendlyNPC",
}

function AloftFrame:OnNameplateShowWithColorByType(aloftData)
	self:OnNameplateShow(aloftData)

	local override = colorMethod(aloftData)
	if #override >= 6 then
		local rhex, ghex, bhex = override:sub(1, 2), override:sub(3, 4), override:sub(5, 6)
		aloftData.backdropFrame:SetBackdropColor(tonumber(rhex, 16)/255, tonumber(ghex, 16)/255, tonumber(bhex, 16)/255, self.db.profile.backgroundAlpha)
	else
		self:SetBackgroundColor(aloftData, self.db.profile.backgroundColors[backgroundTypeColorLookup[aloftData.type]])
	end
end

-----------------------------------------------------------------------------
